package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.PayRecordMapper;
import cn.wisenergy.model.app.PayRecord;
import cn.wisenergy.model.app.User;
import cn.wisenergy.model.dto.PayPageDto;
import cn.wisenergy.model.dto.PayQueryDto;
import cn.wisenergy.model.vo.AddLimitVo;
import cn.wisenergy.service.app.AliPayService;
import cn.wisenergy.service.app.UserLimitService;
import cn.wisenergy.service.common.Common;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.internal.util.AlipaySignature;
import com.alipay.api.request.AlipayTradePagePayRequest;
import com.alipay.api.response.AlipayTradePagePayResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

@Slf4j
@Service
public class AliPayServiceImpl implements AliPayService {
    private static final String CHARSET = "UTF-8";

    @Autowired
    private UserLimitService userLimitService;
    @Autowired
    private PayRecordMapper payRecordMapper;

    @Override
    public R<String> doPost(PayPageDto payPageDto, HttpServletRequest request, HttpServletResponse httpResponse) {
        if (null == payPageDto || null == payPageDto.getTotal()) {
            return R.error("人参不能为空!");
        }

        AlipayClient alipayClient = new DefaultAlipayClient(Common.PAY_URL_CS, Common.APP_ID_CS, Common.PRIVATE_KEY_CS, "json", "UTF-8", Common.ALIPAY_PUBLIC_KEY_CS, "RSA2");
        AlipayTradePagePayRequest alipayRequest = new AlipayTradePagePayRequest();
        String tradeNo = "21" + System.currentTimeMillis();

        //创建API对应的request
        alipayRequest.setReturnUrl("http://domain.com/CallBack/return_url.jsp");
        //在公共参数中设置回跳和通知地址
        alipayRequest.setNotifyUrl("http://x6kmrr.natappfree.cc/pay/aliPayCallBack.do");
        alipayRequest.setBizContent("{" +
                "\"out_trade_no\":\"" + tradeNo + "\"," +
                "\"total_amount\":" + payPageDto.getTotal() + "," +
                "\"subject\":\"充值\"," +
                "\"body\":\"充值\"," +
                "  }");
        String form = "";
        //调用SDK生成表单
        try {
            AlipayTradePagePayResponse response = alipayClient.pageExecute(alipayRequest);
            form = response.getBody();
            httpResponse.setContentType("text/html;charset=UTF-8");
            //直接将完整的表单html输出到页面
            httpResponse.getWriter().write(form);
            httpResponse.getWriter().flush();
            httpResponse.getWriter().close();
            if(response.isSuccess()){
                System.out.println("调用成功");
                //生成预支付订单，充值结果设为失败（result=1）
                PayRecord payRecord = new PayRecord();
                payRecord.setMoney(payPageDto.getTotal());
                payRecord.setResult(1);
                payRecord.setTradeNo(tradeNo);
                payRecord.setType(payPageDto.getPayType());
                payRecord.setUserId(payPageDto.getUserId());
                payRecordMapper.add(payRecord);
                return R.ok(0,form);
            } else {
                System.out.println("调用失败");
                return R.error("调用失败");
            }
        } catch (Exception e) {
            return R.ok(1,"调用失败");
        }
    }

    @Override
    public R<String> query(PayQueryDto payQueryDto) {
        return null;
    }

    @Override
    public R<String> aliPayCallBack(HttpServletResponse response, HttpServletRequest request) {
        System.out.println("支付宝回调成功");
        try {
            //获取支付宝POST过来反馈信息
            Map<String,String> params = new HashMap<String,String>();
            Map requestParams = request.getParameterMap();
            for (Iterator iter = requestParams.keySet().iterator(); iter.hasNext();) {
                String name = (String) iter.next();
                String[] values = (String[]) requestParams.get(name);
                String valueStr = "";
                for (int i = 0; i < values.length; i++) {
                    valueStr = (i == values.length - 1) ? valueStr + values[i]
                            : valueStr + values[i] + ",";
                }
                //乱码解决，这段代码在出现乱码时使用。
                //valueStr = new String(valueStr.getBytes("ISO-8859-1"), "utf-8");
                params.put(name, valueStr);
            }
            boolean flag = AlipaySignature.rsaCheckV1(params,  Common.ALIPAY_PUBLIC_KEY_CS, CHARSET,"RSA2");
            // 获取支付宝的通知返回参数
            String order_no = request.getParameter("out_trade_no"); // 获取订单号
            if (flag==true) {// 验证成功
                try{
                    //返回成功后修改订单充值结果为0,保存查询次数
                    QueryWrapper<PayRecord> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("trade_no",order_no);
                    PayRecord payRecord = payRecordMapper.selectOne(queryWrapper);
                    AddLimitVo addLimitVo = new AddLimitVo();
                    addLimitVo.setTradeNo(order_no);
                    addLimitVo.setPayMoney(payRecord.getMoney());
                    addLimitVo.setPayType(2);
                    addLimitVo.setUserId(payRecord.getUserId());
                    R<Boolean> booleanR = userLimitService.addLimit(addLimitVo);
                    if (booleanR.getCode()==1) {
                        return R.error("用户查询次数添加失败");
                    }
                    return R.ok(0,"充值成功");

                }catch (Exception e)
                {
                    System.out.println("支付宝支付异常");
                    e.printStackTrace();
                    return R.error("支付宝支付异常");
                }
            }
            response.getWriter().write("success");
            return R.ok(0,"支付宝订单支付成功");
        } catch (Exception e) {
            e.printStackTrace();
            return R.error("支付宝支付异常");
        }

    }
}
