package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.utils.Md5Util;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.PayRecordMapper;
import cn.wisenergy.mapper.PriceMapper;
import cn.wisenergy.model.app.PayRecord;
import cn.wisenergy.model.dto.PayPageDto;
import cn.wisenergy.model.vo.AddLimitVo;
import cn.wisenergy.service.app.TestWxPayService;
import cn.wisenergy.service.app.UserLimitService;
import cn.wisenergy.service.util.MapToXmlUtils;
import cn.wisenergy.service.wxpay.WxCommon;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.models.auth.In;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.RandomStringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.DecimalFormat;
import java.util.*;


/**
 * @author chengyan
 */
@Service
@Slf4j
public class TestWxPayServiceImpl implements TestWxPayService {
    public static String basePath = "https://api.mch.weixin.qq.com/pay/unifiedorder";
    @Autowired
    private UserLimitService userLimitService;
    @Autowired
    private PayRecordMapper payRecordMapper;

    /***
     * 支付
     * @param payPageDto
     * @param response
     * @return
     */
    @Override
    public R<String> wxPay(PayPageDto payPageDto, HttpServletResponse response) {
        if(null==payPageDto || null==payPageDto.getTotal()){
            return R.error("入参不能为空");
        }
        ServletOutputStream sos = null;
        String tradeNo = "21" + System.currentTimeMillis();
        //生成预支付订单，充值结果设为失败（result=1）
        PayRecord payRecord = new PayRecord();
        payRecord.setMoney(payPageDto.getTotal());
        payRecord.setResult(1);
        payRecord.setTradeNo(tradeNo);
        payRecord.setType(payPageDto.getPayType());
        payRecord.setUserId(payPageDto.getUserId());
        payRecordMapper.add(payRecord);
        try {
            String orderInfo = createOrderInfo(payPageDto,tradeNo);//生成【统一下单API】所需参数的接口
            System.out.println(orderInfo);
            String code_url = unifiedOrder(orderInfo);//调用统一下单接口
            sos = response.getOutputStream();
            //QRCodeUtil.encode(code_url, sos);//调用生成二维码的方法
            System.out.println("code_url："+code_url);
            return R.ok("0",code_url);
        } catch (Exception e) {
            e.printStackTrace();
            return R.error("获取code_url失败");
        }
    }

    /***
     * 生成统一下单格式的订单，生成一个XML格式的字符串
     * @return
     * @throws UnsupportedEncodingException
     */
    public String createOrderInfo(PayPageDto payPageDto,String tradeNo) throws UnsupportedEncodingException {

        String nonce_str = RandomStringUtils.randomAlphanumeric(16);

        SortedMap<String,String> parameters = new TreeMap<String,String>();
        parameters.put("appid", WxCommon.APP_ID);
        parameters.put("mch_id", WxCommon.MCHID);
        parameters.put("body", "充值");
        parameters.put("out_trade_no", tradeNo);
        parameters.put("nonce_str", nonce_str);
        DecimalFormat df = new DecimalFormat("#");
        parameters.put("total_fee",  df.format((payPageDto.getTotal())*100));
        parameters.put("notify_url",WxCommon.NOTIFY_URL);
        parameters.put("trade_type","NATIVE");
        String characterEncoding = "UTF-8";
        String sign = createSign(characterEncoding,parameters);
        parameters.put("sign", sign);//签名
        //将订单对象转为xml格式
        String s = null;
        try {
            return MapToXmlUtils.mapToXml(parameters);//maptoXml方法是微信sdk自带的方法
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new String(s.getBytes("UTF-8"));
    }

    /***
     * 统一下单
     * @param info
     * @return
     * @throws Exception
     */
    public String unifiedOrder(String info) throws Exception {
        HttpURLConnection conn = (HttpURLConnection) new URL(basePath).openConnection();
        //加入数据
        conn.setRequestMethod("POST");
        conn.setDoOutput(true);

        BufferedOutputStream buffOutStr = new BufferedOutputStream(conn.getOutputStream());
        buffOutStr.write(info.getBytes("UTF-8"));
        buffOutStr.flush();
        buffOutStr.close();

        //获取输入流
        BufferedReader reader = new BufferedReader(new InputStreamReader(conn.getInputStream(), "UTF-8"));

        String line = null;
        StringBuffer sb = new StringBuffer();
        while ((line = reader.readLine()) != null) {
            sb.append(line);
        }
        Map<String, String> map = MapToXmlUtils.xmlToMap(sb.toString());
        //返回的部分字段
        String return_msg = map.get("return_msg");//返回信息
        String return_code = map.get("return_code");//状态码
        String result_code = map.get("result_code");//业务结果
        String code_url = map.get("code_url");
        //根据微信文档return_code 和result_code都为SUCCESS的时候才会返回code_url
        if (null != map && "SUCCESS".equals(return_code) && "SUCCESS".equals(result_code)) {
            return code_url;
        } else {
            System.out.println(return_msg);
            return null;
        }
    }

    /***
     * 生成签名
     * @param characterEncoding
     * @param parameters
     * @return
     */
    public static String createSign(String characterEncoding, SortedMap<String,String> parameters){
        StringBuffer sb = new StringBuffer();
        Set es = parameters.entrySet();//所有参与传参的参数按照accsii排序（升序）
        Iterator it = es.iterator();
        while(it.hasNext()) {
            Map.Entry entry = (Map.Entry)it.next();
            String k = (String)entry.getKey();
            Object v = entry.getValue();
            if(null != v && !"".equals(v)
                    && !"sign".equals(k) && !"key".equals(k)) {
                sb.append(k + "=" + v + "&");
            }
        }
        sb.append("key=" + WxCommon.PRIVATE_KEY);
        String sign = Md5Util.MD5Encode(sb.toString(), characterEncoding).toUpperCase();
        System.out.println("签名："+sign);
        return sign;
    }

    /***
     * 支付成功回调
     * @param request
     * @param response
     * @return
     */
    @Override
    public R<String> wxPayCallBack(HttpServletRequest request, HttpServletResponse response) {
        System.out.println("回调成功");
        try {
            InputStream inStream = request.getInputStream();
            ByteArrayOutputStream outSteam = new ByteArrayOutputStream();
            byte[] buffer = new byte[1024];
            int len = 0;
            while ((len = inStream.read(buffer)) != -1) {
                outSteam.write(buffer, 0, len);
            }
            outSteam.close();
            inStream.close();
            String result = new String(outSteam.toByteArray(), "utf-8");// 获取微信调用我们notify_url的返回信息
            Map<String, String> map = MapToXmlUtils.xmlToMap(result);
            if (map.get("result_code").equalsIgnoreCase("SUCCESS")) {
                //返回成功后修改订单充值结果为0,保存查询次数
                QueryWrapper<PayRecord> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("trade_no",map.get("out_trade_no"));
                PayRecord payRecord = payRecordMapper.selectOne(queryWrapper);
                AddLimitVo addLimitVo = new AddLimitVo();
                addLimitVo.setTradeNo(map.get("out_trade_no"));
                addLimitVo.setPayMoney(payRecord.getMoney());
                addLimitVo.setPayType(2);
                addLimitVo.setUserId(payRecord.getUserId());
                R<Boolean> booleanR = userLimitService.addLimit(addLimitVo);
                if (booleanR.getCode()==1) {
                    return R.error("用户查询次数添加失败");
                }
                return R.ok(0,"充值成功");
            }
            return R.ok(1,"充值失败");
        } catch (Exception e) {
            return R.ok(1,"充值失败");
        }
    }
}
